"""
ScodaKit
=========

A scientific Python-based command-line (CLI) toolkit for S-coda seismic wave analysis and scattering parameters estimation (mean free path and coda attenuation factor).

Author: Marios Karagiorgas
License: Apache 2.0
"""

__version__ = "0.0.1"
__author__ = "Marios Karagiorgas"

